'  Object used for retreiving data from wav files 
'  Copyright 2009 David Sloan  
'  See end of file for terms of use.

con
   WAVE_FORMAT_PCM = $0001
   WAVE_FORMAT_IEEE_FLOAT = $0003
   WAVE_FORMAT_ALAW = $0006
   WAVE_FORMAT_MULAW = $0007
   WAVE_FORMAT_EXTENSIBLE = $fffe
   
var
  byte RIFF_Tag[4]
  long RIFF_Size
  
  byte WAV_ID[4]
  byte WAV_ckid[4]
  long WAV_cksize
  word WAV_FormatTag
  word WAV_nChannels
  long WAV_nSampPerSec
  long WAV_nAvgBytesPerSec
  word WAV_nBlockAlign
  word WAV_wBitsPerSample

  byte DATA_Tag[4]
  long DATA_size

pub new
  pos := 0
  datfound := -1

pub bytesin(Pbt) | i
  if (pos == 0 and datfound == -1)
    repeat i from 0 to 3
      RIFF_Tag[i] := byte[pbt + i]
    if (RIFF_Tag[0] <> "R" or RIFF_Tag[1] <> "I" or RIFF_Tag[2] <> "F" or RIFF_Tag[3] <> "F")
      abort(@ERROR_BadRIFF)
    RIFF_Size := byte[pbt + 4] + byte[pbt + 5]<<8 + byte[pbt + 6]<<16 + byte[pbt + 7]<<24
    repeat i from 8 to 11
      WAV_ID[i - 8] := byte[pbt + i]
    if (WAV_ID[0] <> "W" or WAV_ID[1] <> "A" or WAV_ID[2] <> "V" or WAV_ID[3] <> "E")
      abort(@ERROR_BadWAVE)
    repeat i from 12 to 15
      WAV_ckid[i - 12] := byte[pbt + i]
    if (WAV_ckid[0] <> "f" or WAV_ckid[1] <> "m" or WAV_ckid[2] <> "t" or WAV_ckid[3] <> " ")
      abort(@ERROR_Badfmt)
    WAV_cksize := byte[pbt + 16] + byte[pbt + 17]<<8 + byte[pbt + 18]<<16 + byte[pbt + 19]<<24
    pos := requestlength
  elseif (pos == 20 and datfound == -1)
    WAV_FormatTag := byte[pbt] + byte[pbt + 1]<<8
    if (WAV_FormatTag <> WAVE_FORMAT_PCM and WAV_FormatTag <> WAVE_FORMAT_IEEE_FLOAT and WAV_FormatTag <> WAVE_FORMAT_ALAW and WAV_FormatTag <> WAVE_FORMAT_MULAW and WAV_FormatTag <> WAVE_FORMAT_EXTENSIBLE)
      abort(@ERROR_unrecognizedFormat)
    WAV_nChannels := byte[pbt + 2] + byte[pbt + 3]<<8
    WAV_nSampPerSec :=  byte[pbt + 4] + byte[pbt + 5]<<8 + byte[pbt + 6]<<16 + byte[pbt + 7]<<24
    WAV_nAvgBytesPerSec := byte[pbt + 8] + byte[pbt + 9]<<8 + byte[pbt + 10]<<16 + byte[pbt + 11]<<24
    WAV_nBlockAlign := byte[pbt + 12] + byte[pbt + 13]<<8
    WAV_wBitsPerSample := byte[pbt + 14] + byte[pbt + 15]<<8
    pos += requestlength
  elseif (pos > 20 and datfound => -1)
    repeat i from 0 to 3
      DATA_Tag[i] := byte[pbt + i]
    DATA_size := byte[pbt + 4] + byte[pbt + 5]<<8 + byte[pbt + 6]<<16 + byte[pbt + 7]<<24
    if (DATA_Tag[0] == "d" and DATA_Tag[1] == "a" and DATA_Tag[2] == "t" and DATA_Tag[3] == "a")
      datfound := -2
      pos := 0
    else
      datfound := DATA_Size
    
pub requestlength
if (pos == 0 and datfound == -1)
  return 20
elseif (pos == 20 and datfound == -1)
  return WAV_cksize
elseif (pos > 20 and datfound == -1)
  return 8
elseif (datfound => 0)
  return datfound
else
  return 0

pub WavFormat
  return WAV_FormatTag

pub Channels
  return WAV_nChannels

pub SampleRate
  return WAV_nSampPerSec

pub BitsPerSample
  return WAV_wBitsPerSample

pub DataLen
  return DATA_size

pub FORMAT_PCM
  return WAVE_FORMAT_PCM
pub FORMAT_IEEE_FLOAT
  return WAVE_FORMAT_IEEE_FLOAT
pub FORMAT_ALAW
  return WAVE_FORMAT_ALAW
pub FORMAT_MULAW
  return WAVE_FORMAT_MULAW
pub FORMAT_EXTENSIBLE
  return WAVE_FORMAT_EXTENSIBLE 

dat

pos                             long            0
datfound                        long            -1
ERROR_BadRIFF                   byte            "Bad RIFF Header",0
ERROR_BadWAVE                   byte            "Bad WAVE Header",0
ERROR_Badfmt                    byte            "fmt not found in WAVE chunk",0
ERROR_unrecognizedFormat        byte            "Unrecognized WAVE Format",0

{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}} 